@@ -10,10 +10,12 @@ from datetime import date, datetime, timedelta
 from typing import Dict, List, Optional, Tuple
 
 import alpaca_trade_api as tradeapi
+import nest_asyncio
 import pandas as pd
 import pytz
 from requests.exceptions import HTTPError
 
+from liualgotrader.analytics.analysis import load_trades_by_batch_id
 from liualgotrader.common import config, market_data, trading_data
 from liualgotrader.common.database import create_db_connection
 from liualgotrader.common.decorators import timeit
@@ -57,7 +59,10 @@ starting
 
 def show_usage():
     print(
-        f"usage: {sys.argv[0]} -d SYMBOL -v --batch-list --version --debug-symbol SYMBOL\n"
+        f"usage: {sys.argv[0]} -d SYMBOL -v --batch-list --version --debug-symbol SYMBOL <batch-id>\n"
+    )
+    print(
+        "back-test a trading session, using tradeplan.toml in current directory, given a batch-id (UDID).\n\noptions:\n"
     )
     print("-v, --version\t\tDetailed version details")
     print(
@@ -66,6 +71,9 @@ def show_usage():
     print(
         "--debug-symbol\tWrite verbose debug information for symbol SYMBOL during back-testing"
     )
+    print(
+        "--strict\tRun back-test session only on same symbols traded in the original batch"
+    )
 
 
 def show_version(filename: str, version: str) -> None:
@@ -142,7 +150,10 @@ async def create_strategies(
 
 
 def backtest(
-    batch_id: str, debug_symbols: List[str] = None, conf_dict: Dict = None
+    batch_id: str,
+    debug_symbols: List[str] = None,
+    conf_dict: Dict = None,
+    strict: bool = False,
 ) -> str:
     data_api: tradeapi = tradeapi.REST(
         base_url=config.prod_base_url,
@@ -162,9 +173,11 @@ def backtest(
             symbol: str, scanner_start_time: datetime
         ) -> None:
             est = pytz.timezone("America/New_York")
-            scanner_start_time = pytz.utc.localize(
-                scanner_start_time
-            ).astimezone(est)
+            scanner_start_time = (
+                pytz.utc.localize(scanner_start_time).astimezone(est)
+                if scanner_start_time.tzinfo is None
+                else scanner_start_time
+            )
             start_time = pytz.utc.localize(start).astimezone(est)
 
             if scanner_start_time > start_time + duration:
@@ -182,37 +195,46 @@ def backtest(
             if debug_symbols and symbol in debug_symbols:
                 print("--> using DEBUG mode")
 
-            # load historical data
-            try:
-                symbol_data = data_api.polygon.historic_agg_v2(
-                    symbol,
-                    1,
-                    "minute",
-                    _from=str(start_time - timedelta(days=8)),
-                    to=str(start_time + timedelta(days=1)),
-                    limit=10000,
-                ).df
-            except HTTPError as e:
-                tlog(f"Received HTTP error {e} for {symbol}")
-                return
+            re_try = 3
 
-            if len(symbol_data) < 100:
-                tlog(f"not enough data-points  for {symbol}")
-                return
+            while re_try > 0:
+                # load historical data
+                try:
+                    symbol_data = data_api.polygon.historic_agg_v2(
+                        symbol,
+                        1,
+                        "minute",
+                        _from=str(start_time - timedelta(days=8)),
+                        to=str(start_time + timedelta(days=1)),
+                        limit=10000,
+                    ).df
+                except HTTPError as e:
+                    tlog(f"Received HTTP error {e} for {symbol}")
+                    return
 
-            add_daily_vwap(
-                symbol_data, debug=debug_symbols and symbol in debug_symbols
-            )
-            market_data.minute_history[symbol] = symbol_data
-            print(
-                f"loaded {len(market_data.minute_history[symbol].index)} agg data points"
-            )
+                if len(symbol_data) < 100:
+                    tlog(f"not enough data-points  for {symbol}")
+                    return
 
-            position: int = 0
+                add_daily_vwap(
+                    symbol_data,
+                    debug=debug_symbols and symbol in debug_symbols,
+                )
+                market_data.minute_history[symbol] = symbol_data
+                print(
+                    f"loaded {len(market_data.minute_history[symbol].index)} agg data points"
+                )
+
+                position: int = 0
+                try:
+                    minute_index = symbol_data["close"].index.get_loc(
+                        start_time, method="nearest"
+                    )
+                    break
+                except (Exception, ValueError) as e:
+                    print(f"[EXCEPTION] {e} - trying to reload-data. ")
+                    re_try -= 1
 
-            minute_index = symbol_data["close"].index.get_loc(
-                start_time, method="nearest"
-            )
             new_now = symbol_data.index[minute_index]
             print(f"start time with data {new_now}")
             price = 0.0
@@ -321,10 +343,27 @@ def backtest(
                         str(symbol_data.index[minute_index - 1]),
                     )
 
-        symbols = await TrendingTickers.load(batch_id)
-        print(f"loaded {len(symbols)} symbols")
+        if not strict:
+            symbols_and_start_time = await TrendingTickers.load(batch_id)
 
-        if len(symbols) > 0:
+            num_symbols = len(symbols_and_start_time)
+        else:
+            print("strict mode selected, loading symbols from trades")
+            nest_asyncio.apply()
+            _df = load_trades_by_batch_id(batch_id)
+            symbols = _df.symbol.unique().tolist()
+            num_symbols = len(symbols)
+            est = pytz.timezone("America/New_York")
+            start_time = pytz.utc.localize(_df.start_time.min()).astimezone(
+                est
+            )
+            symbols_and_start_time = list(
+                zip(symbols, [start_time for x in range(num_symbols)])
+            )
+
+        print(f"loaded {len(symbols_and_start_time)} symbols")
+
+        if num_symbols > 0:
             est = pytz.timezone("America/New_York")
             start_time = pytz.utc.localize(start).astimezone(est)
             config.market_open = start_time.replace(
@@ -338,8 +377,10 @@ def backtest(
                 conf_dict, duration, ref_run_id, uid, start  # type: ignore
             )
 
-            for symbol in symbols:
-                await backtest_symbol(symbol[0], symbol[1])
+            for symbol_and_start_time in symbols_and_start_time:
+                await backtest_symbol(
+                    symbol_and_start_time[0], symbol_and_start_time[1]
+                )
 
     @timeit
     async def backtest_worker() -> None:
