@@ -28,6 +28,7 @@ from liualgotrader.models.trending_tickers import TrendingTickers
 from liualgotrader.strategies.base import Strategy, StrategyType
 
 shortable: Dict = {}
+symbol_data_error: Dict = {}
 
 
 async def end_time(reason: str):
@@ -378,6 +379,7 @@ async def handle_data_queue_msg(
     data: Dict, trading_api: tradeapi, data_api: tradeapi
 ) -> bool:
     global shortable
+    global symbol_data_error
 
     symbol = data["symbol"]
     if symbol not in market_data.minute_history:
@@ -570,15 +572,46 @@ async def handle_data_queue_msg(
                 ):
                     continue
 
-                do, what = await s.run(
-                    symbol,
-                    shortable[symbol],
-                    int(symbol_position),
-                    market_data.minute_history[symbol],
-                    ts,
-                    trading_api=trading_api,
-                    portfolio_value=config.portfolio_value,
-                )
+                try:
+                    do, what = await s.run(
+                        symbol,
+                        shortable[symbol],
+                        int(symbol_position),
+                        market_data.minute_history[symbol],
+                        ts,
+                        trading_api=trading_api,
+                        portfolio_value=config.portfolio_value,
+                    )
+                except Exception as e:
+                    tlog(
+                        f"[EXCEPTION] strategy {s.name} for symbol {symbol} -> {e}"
+                    )
+                    exc_info = sys.exc_info()
+                    lines = traceback.format_exception(*exc_info)
+                    for line in lines:
+                        tlog(f"{line}")
+                    del exc_info
+
+                    if symbol not in symbol_data_error:
+                        tlog(f"attempting reload of data for symbol {symbol}")
+
+                        _df = data_api.polygon.historic_agg_v2(
+                            symbol,
+                            1,
+                            "minute",
+                            _from=str(date.today() - timedelta(days=10)),
+                            to=str(date.today() + timedelta(days=1)),
+                        ).df
+                        _df["vwap"] = 0.0
+                        _df["average"] = 0.0
+                        market_data.minute_history[symbol] = _df
+                        tlog(
+                            f"consumer task re-loaded {len(market_data.minute_history[symbol].index)} 1-min candles for {symbol}"
+                        )
+
+                        symbol_data_error[symbol] = True
+
+                    continue
 
                 if do:
                     try:
@@ -792,7 +825,12 @@ async def consumer_async_main(
 
         trading_data.strategies.append(s)
         if symbols:
-            loaded += await load_current_positions(trading_api, symbols, s)
+            loaded += await load_current_positions(
+                trading_api=trading_api,
+                symbols=symbols,
+                strategy=s,
+                env=config.env,
+            )
 
     if symbols and loaded != len(symbols):
         tlog(
@@ -819,7 +857,7 @@ async def consumer_async_main(
 
 
 async def load_current_positions(
-    trading_api: tradeapi, symbols: List[str], strategy: Strategy
+    trading_api: tradeapi, symbols: List[str], strategy: Strategy, env: str
 ) -> int:
     loaded = 0
     for symbol in symbols:
@@ -840,7 +878,10 @@ async def load_current_positions(
                     indicators,
                     timestamp,
                 ) = await NewTrade.load_latest(
-                    config.db_conn_pool, symbol, strategy.name
+                    config.db_conn_pool,
+                    symbol=symbol,
+                    strategy_name=strategy.name,
+                    env=env,
                 )
 
                 if prev_run_id is None:
@@ -915,10 +956,6 @@ def consumer_main(
         asyncio.run(
             consumer_async_main(queue, symbols, unique_id, conf["strategies"])
         )
-        # loop = asyncio.new_event_loop()
-        # asyncio.set_event_loop(asyncio.new_event_loop())
-        # loop.run_until_complete(consumer_async_main(queue, symbols, unique_id))
-        # loop.run_forever()
     except KeyboardInterrupt:
         tlog("consumer_main() - Caught KeyboardInterrupt")
 
